@@ -11,18 +11,18 @@
  ********************************************************************
 
  function: toplevel settings for 44.1/48kHz uncoupled modes
- last mod: $Id: setup_44u.h 13293 2007-07-24 00:09:47Z xiphmont $
+ last mod: $Id: setup_44u.h 15531 2008-11-24 23:50:06Z xiphmont $
 
  ********************************************************************/
 
 #include "modes/residue_44u.h"
 
-static double rate_mapping_44_un[12]={
+static const double rate_mapping_44_un[12]={
   32000.,48000.,60000.,70000.,80000.,86000.,
   96000.,110000.,120000.,140000.,160000.,240001.
 };
 
-ve_setup_data_template ve_setup_44_uncoupled={
+static const ve_setup_data_template ve_setup_44_uncoupled={
   11,
   rate_mapping_44_un,
   quality_mapping_44,
