@@ -11,38 +11,38 @@
  ********************************************************************
 
  function: 8kHz settings 
- last mod: $Id: setup_8.h 13293 2007-07-24 00:09:47Z xiphmont $
+ last mod: $Id: setup_8.h 15531 2008-11-24 23:50:06Z xiphmont $
 
  ********************************************************************/
 
 #include "psych_8.h"
 #include "residue_8.h"
 
-static int blocksize_8[2]={
+static const int blocksize_8[2]={
   512,512
 };
 
-static int _floor_mapping_8[2]={
+static const int _floor_mapping_8[2]={
   6,6,
 };
 
-static double rate_mapping_8[3]={
+static const double rate_mapping_8[3]={
   6000.,9000.,32000.,
 };
 
-static double rate_mapping_8_uncoupled[3]={
+static const double rate_mapping_8_uncoupled[3]={
   8000.,14000.,42000.,
 };
 
-static double quality_mapping_8[3]={
+static const double quality_mapping_8[3]={
   -.1,.0,1.
 };
 
-static double _psy_compand_8_mapping[3]={ 0., 1., 1.};
+static const double _psy_compand_8_mapping[3]={ 0., 1., 1.};
 
-static double _global_mapping_8[3]={ 1., 2., 3. };
+static const double _global_mapping_8[3]={ 1., 2., 3. };
 
-ve_setup_data_template ve_setup_8_stereo={
+static const ve_setup_data_template ve_setup_8_stereo={
   2,
   rate_mapping_8,
   quality_mapping_8,
@@ -93,7 +93,7 @@ ve_setup_data_template ve_setup_8_stereo={
   _mapres_template_8_stereo
 };
 
-ve_setup_data_template ve_setup_8_uncoupled={
+static const ve_setup_data_template ve_setup_8_uncoupled={
   2,
   rate_mapping_8_uncoupled,
   quality_mapping_8,
